package com.qmth.themis.business.cache;

import com.qmth.themis.business.bean.status.ClientWebsocketStatusBean;
import com.qmth.themis.business.bean.status.ExamStatusBean;
import com.qmth.themis.business.bean.status.MonitorCallStatusBean;
import com.qmth.themis.business.bean.status.MonitorStatusBean;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;

import java.util.Objects;

/**
 * 考试记录缓存hash值操作
 *
 * @Description:
 * @Author: xiatian
 * @Date: 2020-07-29
 */
public class ExamRecordCacheUtil {
    private static RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);

    public static Long getId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.id.getCode());
    }

    public static Long getExamStudentId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.exam_student_id.getCode());
    }

    public static Long getPaperId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_id.getCode());
    }

    public static void setFirstStartTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.first_start_time.getCode(), date);
    }

    public static void setFirstPrepareTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.first_prepare_time.getCode(), date);
    }

    public static void setLastPrepareTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_prepare_time.getCode(), date);
    }

    public static void setStatus(Long recordId, ExamRecordStatusEnum status, Long timestamp) {
        ExamStatusBean examStatusBean = getStatusBean(recordId);
        if (Objects.isNull(examStatusBean)) {
            examStatusBean = new ExamStatusBean(status, timestamp);
        } else {
            examStatusBean.setExamRecordStatusEnum(status);
            examStatusBean.setTimestamp(timestamp);
        }
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.status.getCode(), examStatusBean);
    }

    public static Long getFirstStartTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.first_start_time.getCode());
    }

    public static Long getFirstPrepareTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.first_prepare_time.getCode());
    }

    public static Long getLastPrepareTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_prepare_time.getCode());
    }

    public static String getClientCurrentIp(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_current_ip.getCode());
    }

    public static Double getAnswerProgress(Long recordId) {
        return (Double) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.answer_progress.getCode());
    }

    public static ReviewResultEnum getReviewResult(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.review_result.getCode())) ? ReviewResultEnum.valueOf((String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.review_result.getCode())) : null;
    }

    public static ScoreStatusEnum getScoreStatus(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.score_status.getCode())) ? ScoreStatusEnum.valueOf((String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.score_status.getCode())) : null;
    }

    public static Integer getDurationSeconds(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.duration_seconds.getCode());
    }

    public static Double getObjectiveScore(Long recordId) {
        return (Double) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.objective_score.getCode());
    }

    public static Long getExamId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.exam_id.getCode());
    }

    public static String getPaperStructPath(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_struct_path.getCode());
    }

    public static Integer getPaperStructUpload(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_struct_upload.getCode());
    }

    public static Integer getSerialNumber(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.serial_number.getCode());
    }

    public static VerifyExceptionEnum getEntryAuthenticationResult(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_result.getCode())) ? VerifyExceptionEnum.valueOf((String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_result.getCode())) : null;
    }

    public static void setReviewResult(Long recordId, ReviewResultEnum reviewResultEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.review_result.getCode(), reviewResultEnum);
    }

    public static void setScoreStatus(Long recordId, ScoreStatusEnum scoreStatusEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.score_status.getCode(), scoreStatusEnum);
    }

    public static void setPaperStructPath(Long recordId, String paperStructPath) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_struct_path.getCode(), paperStructPath);
    }

    public static void setPaperStructUpload(Long recordId, Integer paperStructUpload) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_struct_upload.getCode(), paperStructUpload);
    }

    public static void setSerialNumber(Long recordId, Integer serialNumber) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.serial_number.getCode(), serialNumber);
    }

    public static void setEntryAuthenticationResult(Long recordId, VerifyExceptionEnum entryAuthenticationResult) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_result.getCode(), entryAuthenticationResult);
    }

    public static void setObjectiveScore(Long recordId, Double objectiveScore) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.objective_score.getCode(), objectiveScore);
    }

    public static void setFinishTime(Long recordId, Long finishTime) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.finish_time.getCode(), finishTime);
    }

    public static Long getFinishTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.finish_time.getCode());
    }

    public static void setDurationSeconds(Long recordId, Integer durationSeconds) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.duration_seconds.getCode(), durationSeconds);
    }

    public static void setFinishType(Long recordId, FinishTypeEnum finishType) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.finish_type.getCode(), finishType);
    }

    public static FinishTypeEnum getFinishType(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.finish_type.getCode())) ? FinishTypeEnum.valueOf((String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.finish_type.getCode())) : null;
    }

    public static Long getClientLastSyncTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_last_sync_time.getCode());
    }

    public static Integer getAlreadyBreakCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.already_break_count.getCode());
    }

    public static void setAlreadyBreakCount(Long recordId, Integer alreadyBreakCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.already_break_count.getCode(), alreadyBreakCount);
    }

    public static ExamStatusBean getStatusBean(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.status.getCode())) ? (ExamStatusBean) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.status.getCode()) : null;
    }

    public static ExamRecordStatusEnum getStatus(Long recordId) {
        ExamStatusBean examStatusBean = getStatusBean(recordId);
        if (Objects.isNull(examStatusBean) || Objects.isNull(examStatusBean.getExamRecordStatusEnum())) {
            throw new BusinessException(ExceptionResultEnum.EXAM_STATUS_NOT_NULL);
        }
        return examStatusBean.getExamRecordStatusEnum();
    }

    public static Long getLastBreakId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_break_id.getCode());
    }

    public static Long getLastBreakTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_break_time.getCode());
    }

    public static void setEntryAuthenticationId(Long recordId, Long entryAuthenticationId) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_id.getCode(), entryAuthenticationId);
    }

    public static Long getExamActivityId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.exam_activity_id.getCode());
    }

    public static void setWarningCount(Long recordId, Integer warningCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.warning_count.getCode(), warningCount);
    }

    public static Integer getWarningCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.warning_count.getCode());
    }

    public static void setBreachStatus(Long recordId, Integer breachStatus) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.breach_status.getCode(), breachStatus);
    }

    public static Integer getBreachStatus(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.breach_status.getCode());
    }

    public static void setInProcessLivenessVerifyCount(Long recordId, Integer inProcessLivenessVerifyCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_verify_count.getCode(), inProcessLivenessVerifyCount);
    }

    public static Integer getInProcessLivenessVerifyCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_verify_count.getCode());
    }

    public static void setMonitorKey(Long recordId, String monitorKey) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.monitor_key.getCode(), monitorKey);
    }

    public static String getMonitorKey(Long recordId) {
        if (Objects.isNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.monitor_key.getCode()))) {
            setMonitorKey(recordId, String.valueOf(redisUtil.getRedisSequence()));
        }
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.monitor_key.getCode());
    }

    public static MonitorStatusSourceEnum getMonitorStatus(Long recordId, MonitorVideoSourceEnum source) {
        MonitorStatusBean monitorStatusBean = getMonitorStatusBean(recordId, source);
        if (Objects.isNull(monitorStatusBean)) {
            return null;
        } else {
            return monitorStatusBean.getMonitorStatusSourceEnum();
        }
    }

    public static MonitorStatusBean getMonitorStatusBean(Long recordId, MonitorVideoSourceEnum source) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_STATUS_ + source.name())) ? (MonitorStatusBean) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_STATUS_ + source.name()) : null;
    }

    public static void setMonitorStatus(Long recordId, MonitorVideoSourceEnum source, MonitorStatusSourceEnum statusSourceEnum, Long timestamp) {
        MonitorStatusBean monitorStatusBean = getMonitorStatusBean(recordId, source);
        if (Objects.isNull(monitorStatusBean)) {
            monitorStatusBean = new MonitorStatusBean(statusSourceEnum, timestamp);
        } else {
            monitorStatusBean.setMonitorStatusSourceEnum(statusSourceEnum);
            monitorStatusBean.setTimestamp(timestamp);
        }
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_STATUS_ + source.name(), monitorStatusBean);
    }

    public static MonitorCallStatusSourceEnum getMonitorCallStatus(Long recordId, MonitorVideoSourceEnum source) {
        MonitorCallStatusBean monitorCallStatusBean = getMonitorCallStatusBean(recordId, source);
        if (Objects.isNull(monitorCallStatusBean)) {
            return null;
        } else {
            return monitorCallStatusBean.getMonitorCallStatusSourceEnum();
        }
    }

    public static MonitorCallStatusBean getMonitorCallStatusBean(Long recordId, MonitorVideoSourceEnum source) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_CALL_STATUS_ + source.name())) ? (MonitorCallStatusBean) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_CALL_STATUS_ + source.name()) : null;
    }

    public static void setMonitorCallStatus(Long recordId, MonitorVideoSourceEnum source, MonitorCallStatusSourceEnum callStatusSourceEnum, Long timestamp) {
        MonitorCallStatusBean monitorCallStatusBean = getMonitorCallStatusBean(recordId, source);
        if (Objects.isNull(monitorCallStatusBean)) {
            monitorCallStatusBean = new MonitorCallStatusBean(callStatusSourceEnum, timestamp);
        } else {
            monitorCallStatusBean.setMonitorCallStatusSourceEnum(callStatusSourceEnum);
            monitorCallStatusBean.setTimestamp(timestamp);
        }
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_CALL_STATUS_ + source.name(), monitorCallStatusBean);
    }

    public static ClientWebsocketStatusBean getClientWebsocketStatusBean(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_websocket_status.getCode())) ? (ClientWebsocketStatusBean) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_websocket_status.getCode()) : null;
    }

    public static WebsocketStatusEnum getClientWebsocketStatus(Long recordId) {
        ClientWebsocketStatusBean clientWebsocketStatusBean = getClientWebsocketStatusBean(recordId);
        if (Objects.isNull(clientWebsocketStatusBean)) {
            return null;
        } else {
            return clientWebsocketStatusBean.getWebsocketStatusEnum();
        }
    }

    public static String getClientWebsocketId(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_websocket_id.getCode());
    }

    public static void setClientWebsocketStatus(Long recordId, WebsocketStatusEnum websocketStatusEnum, Long timestamp) {
        ClientWebsocketStatusBean clientWebsocketStatusBean = getClientWebsocketStatusBean(recordId);
        if (Objects.isNull(clientWebsocketStatusBean)) {
            clientWebsocketStatusBean = new ClientWebsocketStatusBean(websocketStatusEnum, timestamp);
        } else {
            clientWebsocketStatusBean.setWebsocketStatusEnum(websocketStatusEnum);
            clientWebsocketStatusBean.setTimestamp(timestamp);
        }
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_websocket_status.getCode(), clientWebsocketStatusBean);
    }

    public static void setClientCurrentIp(Long recordId, String ip) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_current_ip.getCode(), ip);
    }

    public static void setClientWebsocketId(Long recordId, String id) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_websocket_id.getCode(), id);
    }

    public static void setClientLastSyncTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.client_last_sync_time.getCode(), date);
    }

    public static void setPaperDownload(Long recordId, Integer paperDownload) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_download.getCode(), paperDownload);
    }

    public static void setAnswerProgress(Long recordId, Double progress) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.answer_progress.getCode(), progress);
    }

    public static void setLastBreakId(Long recordId, Long breakId) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_break_id.getCode(), breakId);
    }

    public static void setLastBreakTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_break_time.getCode(), date);
    }

    public static void setLastStartTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_start_time.getCode(), date);
    }

    public static Long getLastStartTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.last_start_time.getCode());
    }

    public static Integer getPaperDownload(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.paper_download.getCode());
    }

    public static Long getStartTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.start_time.getCode());
    }

    public static Long getEndTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.end_time.getCode());
    }

    public static Integer getOpeningSeconds(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.opening_seconds.getCode());
    }

    public static Integer getMinDurationSeconds(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.min_duration_seconds.getCode());
    }

    public static Integer getMaxDurationSeconds(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.max_duration_seconds.getCode());
    }

    public static Integer getForceFinish(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.force_finish.getCode());
    }

    public static void setStartTime(Long recordId, Long startTime) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.start_time.getCode(), startTime);
    }

    public static void setEndTime(Long recordId, Long endTime) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.end_time.getCode(), endTime);
    }

    public static void setOpeningSeconds(Long recordId, Integer openingSeconds) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.opening_seconds.getCode(), openingSeconds);
    }

    public static void setMinDurationSeconds(Long recordId, Integer minDurationSeconds) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.min_duration_seconds.getCode(), minDurationSeconds);
    }

    public static void setMaxDurationSeconds(Long recordId, Integer maxDurationSeconds) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.max_duration_seconds.getCode(), maxDurationSeconds);
    }

    public static void setForceFinish(Long recordId, Integer forceFinish) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.force_finish.getCode(), forceFinish);
    }

    public static void setLastPrepareTime(Long recordId, ScoreStatusEnum scoreStatusEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.score_status.getCode(), scoreStatusEnum);
    }

    public static void setExamFinalFinishTime(Long recordId, Long date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "finalFinishTime", date);
    }

    public static Long getExamFinalFinishTime(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "finalFinishTime");
    }

    public static void setMultipleFaceCountErrorRandom(Long recordId, String multipleFaceCountErrorRandom) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "multipleFaceCountErrorRandom", multipleFaceCountErrorRandom);
    }

    public static void setRealnessErrorRandom(Long recordId, String realnessErrorRandom) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "realnessErrorRandom", realnessErrorRandom);
    }

    public static void setNoFaceCountErrorRandom(Long recordId, String noFaceCountErrorRandom) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "noFaceCountErrorRandom", noFaceCountErrorRandom);
    }

    public static void setFaceCompareErrorRandom(Long recordId, String faceCompareErrorRandom) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "faceCompareErrorRandom", faceCompareErrorRandom);
    }

    public static void setEyeCloseErrorRandom(Long recordId, String eyeCloseErrorRandom) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "eyeCloseErrorRandom", eyeCloseErrorRandom);
    }

    public static void setMultipleFaceCountErrorNum(Long recordId, Integer multipleFaceCountErrorNum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "multipleFaceCountErrorNum", multipleFaceCountErrorNum);
    }

    public static void setNoFaceCountErrorNum(Long recordId, Integer noFaceCountErrorNum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "noFaceCountErrorNum", noFaceCountErrorNum);
    }

    public static void setFaceCompareErrorNum(Long recordId, Integer faceCompareErrorNum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "faceCompareErrorNum", faceCompareErrorNum);
    }

    public static void setEyeCloseErrorNum(Long recordId, Integer eyeCloseErrorNum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "eyeCloseErrorNum", eyeCloseErrorNum);
    }

    public static void setRealnessErrorNum(Long recordId, Integer realnessErrorNum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "realnessErrorNum", realnessErrorNum);
    }

    public static String getMultipleFaceCountErrorRandom(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "multipleFaceCountErrorRandom");
    }

    public static String getNoFaceCountErrorRandom(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "noFaceCountErrorRandom");
    }

    public static String getFaceCompareErrorRandom(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "faceCompareErrorRandom");
    }

    public static String getEyeCloseErrorRandom(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "eyeCloseErrorRandom");
    }

    public static String getRealnessErrorRandom(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "realnessErrorRandom");
    }

    public static Integer getMultipleFaceCountErrorNum(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "multipleFaceCountErrorNum")) ? (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "multipleFaceCountErrorNum") : 0;
    }

    public static Integer getNoFaceCountErrorNum(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "noFaceCountErrorNum")) ? (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "noFaceCountErrorNum") : 0;
    }

    public static Integer getFaceCompareErrorNum(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "faceCompareErrorNum")) ? (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "faceCompareErrorNum") : 0;
    }

    public static Integer getEyeCloseErrorNum(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "eyeCloseErrorNum")) ? (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "eyeCloseErrorNum") : 0;
    }

    public static Integer getRealnessErrorNum(Long recordId) {
        return Objects.nonNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "realnessErrorNum")) ? (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "realnessErrorNum") : 0;
    }

    public static Integer getHasAnswerFile(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.has_answer_file.getCode());
    }

    public static void setCameraPhotoUpload(Long recordId, Integer cameraPhotoUpload) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.camera_photo_upload.getCode(), cameraPhotoUpload);
    }

    public static void setMobilePhotoUpload(Long recordId, Integer mobilePhotoUpload) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.mobile_photo_upload.getCode(), mobilePhotoUpload);
    }

    public static void setEntryAuthenticationPolicy(Long recordId, String entryAuthenticationPolicy) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_policy.getCode(), entryAuthenticationPolicy);
    }

    public static void setInProcessFaceStrangerIgnore(Long recordId, Integer inProcessFaceStrangerIgnore) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_face_stranger_ignore.getCode(), inProcessFaceStrangerIgnore);
    }

    public static void setInProcessLivenessFixedRange(Long recordId, String inProcessLivenessFixedRange) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_fixed_range.getCode(), inProcessLivenessFixedRange);
    }

    public static void setInProcessLivenessJudgePolicy(Long recordId, String inProcessLivenessJudgePolicy) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_judge_policy.getCode(), inProcessLivenessJudgePolicy);
    }

    public static void setMonitorVideoSource(Long recordId, String monitorVideoSource) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.monitor_video_source.getCode(), monitorVideoSource);
    }

    public static void setInProcessRealnessVerifyStatus(Long recordId, Integer inProcessRealnessVerifyStatus) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_realness_verify_status.getCode(), inProcessRealnessVerifyStatus);
    }

    public static void setInProcessLivenessVerifyStatus(Long recordId, Integer inProcessLivenessVerifyStatus) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_verify_status.getCode(), inProcessLivenessVerifyStatus);
    }

    public static void setInProcessFaceVerifyStatus(Long recordId, Integer inProcessFaceVerifyStatus) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_face_verify_status.getCode(), inProcessFaceVerifyStatus);
    }

    public static Integer getCameraPhotoUpload(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.camera_photo_upload.getCode());
    }

    public static Integer getMobilePhotoUpload(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.mobile_photo_upload.getCode());
    }

    public static Long getEntryAuthenticationId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_id.getCode());
    }

    public static String getEntryAuthenticationPolicy(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.entry_authentication_policy.getCode());
    }

    public static Integer getInProcessFaceStrangerIgnore(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_face_stranger_ignore.getCode());
    }

    public static String getInProcessLivenessFixedRange(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_fixed_range.getCode());
    }

    public static String getInProcessLivenessJudgePolicy(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_judge_policy.getCode());
    }

    public static String getMonitorVideoSource(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.monitor_video_source.getCode());
    }

    public static Integer getInProcessRealnessVerifyStatus(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_realness_verify_status.getCode());
    }

    public static Integer getInProcessLivenessVerifyStatus(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_liveness_verify_status.getCode());
    }

    public static Integer getInProcessFaceVerifyStatus(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), ExamRecordFieldEnum.in_process_face_verify_status.getCode());
    }

    /**
     * 更新考试记录缓存
     *
     * @param recordId
     * @param examCache
     * @param ac
     */
    public static void updateExamRecordCache(Long recordId, ExamCacheBean examCache, ExamActivityCacheBean ac) {
        ExamRecordCacheUtil.setStartTime(recordId, ac.getStartTime());
        ExamRecordCacheUtil.setEndTime(recordId, ac.getFinishTime());
        ExamRecordCacheUtil.setOpeningSeconds(recordId, ac.getOpeningSeconds());
        ExamRecordCacheUtil.setMinDurationSeconds(recordId, examCache.getMinDurationSeconds());
        ExamRecordCacheUtil.setMaxDurationSeconds(recordId, ac.getMaxDurationSeconds());
        ExamRecordCacheUtil.setForceFinish(recordId, examCache.getForceFinish());
        ExamRecordCacheUtil.setCameraPhotoUpload(recordId, examCache.getCameraPhotoUpload());
        ExamRecordCacheUtil.setMobilePhotoUpload(recordId, examCache.getMobilePhotoUpload());
        ExamRecordCacheUtil.setEntryAuthenticationPolicy(recordId, examCache.getEntryAuthenticationPolicy().name());
        ExamRecordCacheUtil.setInProcessRealnessVerifyStatus(recordId, examCache.getInProcessRealnessVerify());
        ExamRecordCacheUtil.setInProcessFaceStrangerIgnore(recordId, examCache.getInProcessFaceStrangerIgnore());
        ExamRecordCacheUtil.setInProcessLivenessJudgePolicy(recordId, examCache.getInProcessLivenessJudgePolicy().name());
        ExamRecordCacheUtil.setInProcessLivenessFixedRange(recordId, examCache.getInProcessLivenessFixedRange());
    }
}
