package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamCourseCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.dao.TEExamActivityMapper;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.dto.response.StreamDto;
import com.qmth.themis.business.dto.response.TEExamActivityDto;
import com.qmth.themis.business.dto.response.TEExamActivityQueryDto;
import com.qmth.themis.business.dto.response.TEExamActivityWaitDto;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.entity.TEExam;
import com.qmth.themis.business.entity.TEExamActivity;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.*;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description: 考试场次 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamActivityServiceImpl extends ServiceImpl<TEExamActivityMapper, TEExamActivity> implements TEExamActivityService {

    @Resource
    private TEExamActivityMapper teExamActivityMapper;

    @Resource
    private TEExamService teExamService;

    @Resource
    private TEExamStudentService teExamStudentService;

    @Resource
    private TEExamCourseService teExamCourseService;

    @Resource
    private MqDtoService mqDtoService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private UidUtil uidUtil;

    @Resource
    private MqUtil mqUtil;

    /**
     * 表是否存在
     *
     * @param tableName
     * @param schema
     * @return
     */
    @Override
    public Integer existTable(String tableName, String schema) {
        return teExamActivityMapper.existTable(tableName, schema);
    }

    /**
     * 删除表
     *
     * @param tableName
     */
    @Override
    public void dropTable(String tableName) {
        teExamActivityMapper.dropTable(tableName);
    }

    /**
     * 创建表
     *
     * @param tableName
     */
    @Override
    public void createNewTable(String tableName) {
        teExamActivityMapper.createNewTable(tableName);
    }

    /**
     * 新插数据
     *
     * @param tableName
     * @param teExamActivity
     */
    @Override
    public void insertInfo(String tableName, TEExamActivity teExamActivity) {
        teExamActivityMapper.insertInfo(tableName, teExamActivity);
    }

    /**
     * 查询所有数据
     *
     * @param iPage
     * @param tableName
     * @return
     */
    @Override
    public IPage<TEExamActivity> selectListPage(IPage<Map> iPage, String tableName) {
        return teExamActivityMapper.selectListPage(iPage, tableName);
    }

    /**
     * 查询考场场次信息
     *
     * @param iPage
     * @param examId
     * @param code
     * @param startDate
     * @param finishDate
     * @return
     */
    @Override
    public IPage<TEExamActivityQueryDto> examActivityQuery(IPage<Map> iPage, Long id, Long examId, String code, Long startDate, Long finishDate) {
        return teExamActivityMapper.examActivityQuery(iPage, id, examId, code, startDate, finishDate);
    }

    /**
     * 获取考试待考列表
     *
     * @param studentId
     * @param examActivityIds
     * @param mode
     * @return
     */
    @Override
    public List<TEExamActivityWaitDto> getWaitingExam(Long studentId, Set<Long> examActivityIds, String mode) {
        return teExamActivityMapper.getWaitingExam(studentId, examActivityIds, mode);
    }

    /**
     * 获取考试待考列表
     *
     * @param examActivityId
     * @param examStudentId
     * @param courseCode
     * @param prefix
     * @return
     */
    @Override
    public TEExamActivityDto getWaitingExam(Long examActivityId, Long examStudentId, String courseCode, String prefix, Long recordId) {
        TEExamActivityDto teExamActivityDto = teExamActivityMapper.getWaitingExamByExamActivityId(examActivityId);
        ExamCacheBean examCache = teExamService.getExamCacheBean(teExamActivityDto.getExamId());
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService
                .getExamStudentCacheBean(examStudentId);
        teExamActivityDto.setLeftExamCount(
                (examCache.getExamCount().intValue() - (examStudentCacheBean.getAlreadyExamCount().intValue())
                        < 0 ?
                        0 :
                        examCache.getExamCount().intValue() - (examStudentCacheBean.getAlreadyExamCount()
                                .intValue())));
        if (Objects.nonNull(teExamActivityDto.getInProcessLivenessFixedRangeStr())) {
            String[] longs = teExamActivityDto.getInProcessLivenessFixedRangeStr().trim().replaceAll(" ", "").split(",");
            List inProcessLivenessFixedRange = new ArrayList();
            for (int i = 0; i < longs.length; i++) {
                Integer integer = Integer.valueOf(longs[i].trim());
                inProcessLivenessFixedRange.add(integer);
            }
            if (Objects.equals(inProcessLivenessFixedRange.toString().trim().replaceAll(" ", ""), "")) {
                teExamActivityDto.setInProcessLivenessFixedRange(null);
            } else {
                teExamActivityDto.setInProcessLivenessFixedRange(inProcessLivenessFixedRange);
            }
        }
        if (Objects.nonNull(teExamActivityDto.getMonitorVideoSourceStr()) && !Objects
                .equals(teExamActivityDto.getMonitorVideoSourceStr().toString().trim().replaceAll(" ", ""), "")) {
            List<String> videoSources = Arrays.asList(teExamActivityDto.getMonitorVideoSourceStr().trim().toUpperCase().replaceAll(" ", "").split(","));
            List<StreamDto> streamDtoList = new ArrayList<>();
            videoSources.forEach(s -> {
                streamDtoList.add(new StreamDto(MonitorVideoSourceEnum.valueOf(s), prefix + "_" + recordId + "_" + MonitorVideoSourceEnum.valueOf(s).name().toLowerCase()));
            });
            teExamActivityDto.setMonitorVideoSource(streamDtoList);
            teExamActivityDto.setMonitorAudioEnable(examCache.getMonitorAudioEnable());
            //加入hardwareTest逻辑
            if (teExamActivityDto.getMonitorVideoSourceStr().toUpperCase().contains(MonitorVideoSourceEnum.CLIENT_CAMERA.name())
                    || (Objects.nonNull(teExamActivityDto.getEntryAuthenticationPolicy()) && (
                    Objects.equals(teExamActivityDto.getEntryAuthenticationPolicy(),
                            EntryAuthenticationPolicyEnum.LIVENESS_VERIFY.name()) || Objects
                            .equals(teExamActivityDto.getEntryAuthenticationPolicy(),
                                    EntryAuthenticationPolicyEnum.FACE_VERIFY_FORCE.name()))) || (
                    Objects.nonNull(teExamActivityDto.getCameraPhotoUpload()) && teExamActivityDto.getCameraPhotoUpload() == 1)) {
                List<String> hardwareTest = teExamActivityDto.getHardwareTest();
                if (Objects.isNull(hardwareTest)) {
                    hardwareTest = new ArrayList<>();
                }
                hardwareTest.add(HardwareTestEnum.CAMERA.name());
                //取course缓存
                ExamCourseCacheBean examCourseCacheBean = teExamCourseService
                        .getExamCourseCacheBean(teExamActivityDto.getExamId(), courseCode);
                if (Objects.nonNull(examCourseCacheBean) && Objects.nonNull(examCourseCacheBean.getHasAudio())
                        && examCourseCacheBean.getHasAudio() == 1) {
                    hardwareTest.add(HardwareTestEnum.AUDIOPLAY.name());
                }
                teExamActivityDto.setHardwareTest(hardwareTest);
            }
        } else {
            teExamActivityDto.setMonitorVideoSource(null);
        }
        return teExamActivityDto;
    }

    @Cacheable(value = "exam_activity", key = "#examActivityId", unless = "#result == null")
    @Override
    public ExamActivityCacheBean getExamActivityCacheBean(Long examActivityId) {
        return cacheOperation(examActivityId);
    }

    /**
     * 更新场次缓存
     *
     * @param examActivityId
     * @return
     */
    @Override
    @CachePut(value = "exam_activity", key = "#examActivityId", condition = "#result != null")
    public ExamActivityCacheBean updateExamActivityCacheBean(Long examActivityId) {
        return cacheOperation(examActivityId);
    }

    /**
     * 删除场次缓存
     *
     * @param examActivityId
     */
    @Override
    @CacheEvict(value = "exam_activity", key = "#examActivityId")
    public void deleteExamActivityCacheBean(Long examActivityId) {

    }

    /**
     * 缓存操作
     *
     * @param examActivityId
     * @return
     */
    public ExamActivityCacheBean cacheOperation(Long examActivityId) {
        ExamActivityCacheBean ret = null;
        TEExamActivity ac = getById(examActivityId);
        if (ac == null) {
            return ret;
        }
        if (ac.getEnable() != null && ac.getEnable().intValue() == 0) {
            return ret;
        }
        ret = new ExamActivityCacheBean();
        ret.setId(ac.getId());
        ret.setCode(ac.getCode());
        ret.setExamId(ac.getExamId());
        ret.setPrepareSeconds(ac.getPrepareSeconds());
        ret.setMaxDurationSeconds(ac.getMaxDurationSeconds());
        ret.setOpeningSeconds(ac.getOpeningSeconds());
        ret.setStartTime(ac.getStartTime());
        ret.setFinishTime(ac.getFinishTime());
        ret.setEnable(ac.getEnable());
        return ret;
    }

    @Override
    public List<TEExamActivity> findByExamIdAndOrgId(Long examId, Long orgId) {
        return teExamActivityMapper.findByExamIdAndOrgId(examId, orgId);
    }

    @Transactional
    @Override
    public void saveExamActivity(List<TEExamActivity> teExamActivityList) {
        if (Objects.isNull(teExamActivityList) || teExamActivityList.size() == 0) {
            throw new BusinessException(ExceptionResultEnum.EXAM_INFO_IS_NULL);
        }
        Long examId = null;
        try {
            TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
            examId = teExamActivityList.get(0).getExamId();
            TEExam teExam = teExamService.getById(teExamActivityList.get(0).getExamId());
            if (Objects.nonNull(teExam.getMonitorStatus()) && Objects
                    .equals(teExam.getMonitorStatus(), InvigilateMonitorStatusEnum.FINISHED)) {
                throw new BusinessException("监考结束的考试场次不可以修改");
            }
            teExamActivityList.forEach(s -> {
                if (Objects.nonNull(s.getId())) {
                    s.setUpdateId(tbUser.getId());
                } else {
                    s.setId(uidUtil.getId());
                    s.setCreateId(tbUser.getId());
                    s.setCode(String.valueOf(redisUtil.getRedisActivityCodeSequence(s.getExamId())));
                }
                this.saveOrUpdate(s);
            });
            for (TEExamActivity ac : teExamActivityList) {
                this.updateExamActivityCacheBean(ac.getId());
            }
            if (Objects.nonNull(teExam.getForceFinish()) && teExam.getForceFinish().intValue() == 1) {
                //新增quartz任务,发送mq消息start
                Map<String, Object> prop = new HashMap<>();
                prop.put("oper", "insert");
                prop.put("exam", teExam);
                MqDto mqDto = new MqDto(mqUtil.getMqGroupDomain().getTopic(), MqTagEnum.EXAM_ACTIVITY.name(),
                        JacksonUtil.parseJson(teExamActivityList), MqTagEnum.EXAM_ACTIVITY,
                        String.valueOf(teExam.getId()), prop, tbUser.getName());
                mqDtoService.assembleSendOneWayMsg(mqDto);
                //新增quartz任务,发送mq消息end
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            if (Objects.nonNull(examId) && Objects.nonNull(teExamActivityList) && (Objects.nonNull(teExamActivityList)
                    && teExamActivityList.size() > 0 && Objects.nonNull(teExamActivityList.get(0)))) {
                redisUtil.setRedisActivityCodeSequence(examId,
                        Integer.parseInt(teExamActivityList.get(0).getCode()) - 1);
            }
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length())
                        .replaceAll("'", "");
                throw new BusinessException("机构id[" + teExamActivityList.get(0).getExamId() + "]下的" + FieldUniqueEnum
                        .convertToCode(columnStr) + "数据不允许重复插入");
            } else if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public List<Long> findExamIdToday() {
        try {
            Date now = new Date();
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            String today = sdf1.format(now);
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Long start = sdf2.parse(today + " 00:00:00").getTime();
            Long end = sdf2.parse(today + " 23:59:59").getTime();
            return teExamActivityMapper.findExamIdToday(start, end);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;
    }
}
