package com.qmth.themis.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.dto.WarningDto;
import com.qmth.themis.business.entity.TEConfig;
import com.qmth.themis.business.entity.TEExamStudentLog;
import com.qmth.themis.business.entity.TIeInvigilateWarnInfo;
import com.qmth.themis.business.enums.ExamTypeEnum;
import com.qmth.themis.business.enums.VerifyExceptionEnum;
import com.qmth.themis.business.enums.WarningLevelEnum;
import com.qmth.themis.business.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description: 预警规则公用
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/8/18
 */
@Service
public class WarningServiceImpl implements WarningService {
    private final static Logger log = LoggerFactory.getLogger(WarningServiceImpl.class);

    @Resource
    TEConfigService teConfigService;

    @Resource
    TOeFaceVerifyHistoryService faceVerifyHistoryService;

    @Resource
    TIeInvigilateWarnInfoService tIeInvigilateWarnInfoService;

    @Resource
    TEExamStudentLogService teExamStudentLogService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TOeExamRecordService tOeExamRecordService;

    /**
     * 人脸数量异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void faceCountError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        Integer faceCount = warningDto.getFaceCount();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        if (faceCount > 1) {//多张人脸
            Map<String, Object> map = faceVerifyHistoryService.faceCountError(recordId, warningDto.getWarningEnum().name(), faceCount, true, ExamTypeEnum.IN_PROCESS.name(), warningDto.getRandom());
            if (Objects.nonNull(map) && map.size() > 0) {
                Integer count = Integer.parseInt(String.valueOf(map.get("tmpCount")));
                count = Objects.isNull(count) ? 0 : count;
                if (count > teConfig.getMultipleFaceCountError()) {
                    TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(1), WarningLevelEnum.valueOf(warningEnum.getLevel().get(1)).getTitle(), warningEnum, photoUrl, Objects.nonNull(ExamRecordCacheUtil.getBreachStatus(recordId)) ? ExamRecordCacheUtil.getBreachStatus(recordId) : 1);
                    tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                    this.setWarningCount(recordId);
                    this.setPhotoUrls(map, tIeInvigilateWarnInfo, examStudentCacheBean, recordId, teConfig.getMultipleFaceCountError());
                }
            }
        } else if (faceCount <= 0) {//未检测到人脸
            Map<String, Object> map = faceVerifyHistoryService.faceCountError(recordId, warningDto.getWarningEnum().name(), faceCount, false, ExamTypeEnum.IN_PROCESS.name(), warningDto.getRandom());
            if (Objects.nonNull(map) && map.size() > 0) {
                Integer count = Integer.parseInt(String.valueOf(map.get("tmpCount")));
                count = Objects.isNull(count) ? 0 : count;
                if (count > teConfig.getNoFaceCountError()) {
                    TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(0), WarningLevelEnum.valueOf(warningEnum.getLevel().get(0)).getTitle(), warningEnum, photoUrl, Objects.nonNull(ExamRecordCacheUtil.getBreachStatus(recordId)) ? ExamRecordCacheUtil.getBreachStatus(recordId) : 1);
                    tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                    this.setWarningCount(recordId);
                    this.setPhotoUrls(map, tIeInvigilateWarnInfo, examStudentCacheBean, recordId, teConfig.getNoFaceCountError());
                }
            }
        }
    }

    /**
     * 人脸比对异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void faceCompareError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        Map<String, Object> map = faceVerifyHistoryService.faceCompareError(recordId, warningDto.getWarningEnum().name(), ExamTypeEnum.IN_PROCESS.name(), warningDto.getRandom());
        if (Objects.nonNull(map) && map.size() > 0) {
            Integer count = Integer.parseInt(String.valueOf(map.get("tmpCount")));
            count = Objects.isNull(count) ? 0 : count;
            if (count > teConfig.getTotalFaceCompareErrorCount()) {
                TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(0), WarningLevelEnum.valueOf(warningEnum.getLevel().get(0)).getTitle(), warningEnum, photoUrl, Objects.nonNull(ExamRecordCacheUtil.getBreachStatus(recordId)) ? ExamRecordCacheUtil.getBreachStatus(recordId) : 1);
                tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                this.setWarningCount(recordId);
                this.setPhotoUrls(map, tIeInvigilateWarnInfo, examStudentCacheBean, recordId, teConfig.getTotalFaceCompareErrorCount());
            }
        }
    }

    /**
     * 闭眼检测异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void eyeCloseError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        Map<String, Object> map = faceVerifyHistoryService.eyeCloseError(recordId, warningDto.getWarningEnum().name(), ExamTypeEnum.IN_PROCESS.name(), warningDto.getRandom());
        if (Objects.nonNull(map) && map.size() > 0) {
            Integer count = Integer.parseInt(String.valueOf(map.get("tmpCount")));
            count = Objects.isNull(count) ? 0 : count;
            if (count > teConfig.getTotalEyeCloseErrorCount()) {
                TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(0), WarningLevelEnum.valueOf(warningEnum.getLevel().get(0)).getTitle(), warningEnum, photoUrl, Objects.nonNull(ExamRecordCacheUtil.getBreachStatus(recordId)) ? ExamRecordCacheUtil.getBreachStatus(recordId) : 1);
                tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                this.setWarningCount(recordId);
                this.setPhotoUrls(map, tIeInvigilateWarnInfo, examStudentCacheBean, recordId, 3);
            }
        }
    }

    /**
     * 活体动作异常
     *
     * @param warningDto
     */
    @Override
    public void livenessActionError(WarningDto warningDto) {

    }

    /**
     * 真实性异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void realnessError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
        Map<String, Object> map = faceVerifyHistoryService.realnessError(recordId, warningDto.getWarningEnum().name(), ExamTypeEnum.IN_PROCESS.name(), warningDto.getRandom());
        if (Objects.nonNull(map) && map.size() > 0) {
            Integer count = Integer.parseInt(String.valueOf(map.get("tmpCount")));
            count = Objects.isNull(count) ? 0 : count;
            if (count > teConfig.getRealnessCount()) {
                TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, WarningLevelEnum.D15.name(), WarningLevelEnum.D15.getTitle(), warningEnum, photoUrl, Objects.nonNull(ExamRecordCacheUtil.getBreachStatus(recordId)) ? ExamRecordCacheUtil.getBreachStatus(recordId) : 1);
                tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                this.setWarningCount(recordId);
                this.setPhotoUrls(map, tIeInvigilateWarnInfo, examStudentCacheBean, recordId, teConfig.getRealnessCount());
            }
        }
    }

    /**
     * 设置预警数
     *
     * @param recordId
     */
    public void setWarningCount(Long recordId) {
        Integer warningCount = ExamRecordCacheUtil.getWarningCount(recordId);
        if (Objects.nonNull(warningCount)) {
            warningCount++;
            ExamRecordCacheUtil.setWarningCount(recordId, warningCount);
        } else {
            ExamRecordCacheUtil.setWarningCount(recordId, 1);
        }
        tOeExamRecordService.sendExamRecordDataSaveMq(recordId, System.currentTimeMillis());
    }

    /**
     * 设置图片集合
     *
     * @param map
     * @param tIeInvigilateWarnInfo
     * @param examStudentCacheBean
     * @param recordId
     * @param count
     */
    public void setPhotoUrls(Map<String, Object> map, TIeInvigilateWarnInfo tIeInvigilateWarnInfo, ExamStudentCacheBean examStudentCacheBean, Long recordId, Integer count) {
        List<String> photoUrls = new ArrayList();
        if (Objects.nonNull(map.get("photoUrls")) && !Objects.equals(map.get("photoUrls"), "")) {
            String[] s = String.valueOf(map.get("photoUrls")).split(",");
            Collections.addAll(photoUrls, s);
            if (photoUrls.size() > count) {
                photoUrls = photoUrls.subList(0, count);
            }
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("PHOTOS", photoUrls);
        jsonObject.put("MIN_CREATE_TIME", map.get("createTime"));
        TEExamStudentLog teExamStudentLog = new TEExamStudentLog(tIeInvigilateWarnInfo.getType().name(), WarningLevelEnum.valueOf(tIeInvigilateWarnInfo.getLevel()).getTitle(), jsonObject.toJSONString(), examStudentCacheBean.getStudentId(), examStudentCacheBean.getId(), recordId, WarningLevelEnum.valueOf(tIeInvigilateWarnInfo.getLevel()).getSecondTitle());
        teExamStudentLogService.saveOrUpdate(teExamStudentLog);
    }
}
