package com.qmth.themis.exam.listener.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.google.gson.Gson;
import com.qmth.themis.business.bean.status.MonitorStatusBean;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.dto.WebsocketDto;
import com.qmth.themis.business.entity.TEExamStudentLog;
import com.qmth.themis.business.entity.TIeExamInvigilateNotice;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.TEExamStudentLogService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.service.TIeExamInvigilateNoticeService;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.exam.listener.service.MqOeLogicService;
import com.qmth.themis.exam.websocket.WebSocketMobileServer;
import com.qmth.themis.exam.websocket.WebSocketOeServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * @Description: mq 考生端执行逻辑 impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class MqOeLogicServiceImpl implements MqOeLogicService {
    private final static Logger log = LoggerFactory.getLogger(MqOeLogicServiceImpl.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    TMRocketMessageService tmRocketMessageService;

    @Resource
    TEExamStudentLogService teExamStudentLogService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TIeExamInvigilateNoticeService tIeExamInvigilateNoticeService;

    /**
     * oe逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqOeLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        ConcurrentHashMap<String, WebSocketOeServer> webSocketMap = WebSocketOeServer.getWebSocketMap();
        String tag = mqDto.getTag();
//        AtomicBoolean match = new AtomicBoolean(false);
        if (Objects.equals(MqTagEnum.OE_MONITOR_FINISH.name(), tag)) {//强制离线交卷
            Set examRecordId = JacksonUtil.readJson(String.valueOf(mqDto.getBody()), Set.class);
            examRecordId.forEach(s -> {
                Long recordId = Long.parseLong(String.valueOf(s));
                //获取考试记录的客户端websocketId
                String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
                if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId))) {
                    Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                    ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
                    WebSocketOeServer webSocketOeServer = webSocketMap.get(clientWebsocketId);
                    ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                    if ((Objects.nonNull(examRecordStatusEnum)
                            && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                            && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                            && Objects.nonNull(webSocketOeServer.getRecordId())
                            && webSocketOeServer.getRecordId().longValue() == recordId.longValue()) {
//                        match.set(true);
                        Map map = new HashMap<>();
                        map.put("form", mqDto.getObjName());
                        map.put(SystemConstant.RECORD_ID, recordId);
                        map.put(SystemConstant.MESSAGE, FinishTypeEnum.valueOf(String.valueOf(mqDto.getProperties().get("type"))).getCode());
                        WebsocketDto websocketDto = new WebsocketDto(WebsocketTypeEnum.INVIGILATE_STOP_EXAM.name(), map);
                        webSocketOeServer.sendMessage(websocketDto);
                        TEExamStudentLog teExamStudentLog = new TEExamStudentLog(mqDto.getType().name(), mqDto.getType().getCode(), mqDto.getType().getCode(), examStudentCacheBean.getStudentId(), examStudentId, recordId, mqDto.getType().getCode());
                        teExamStudentLogService.save(teExamStudentLog);
                    }
                }
            });
        } else if (Objects.equals(MqTagEnum.OE_WARNING_FINISH.name(), tag)) {//预警交卷
            Set examRecordId = JacksonUtil.readJson(String.valueOf(mqDto.getBody()), Set.class);
            examRecordId.forEach(s -> {
                Long recordId = Long.parseLong(String.valueOf(s));
                String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
                if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId))) {
                    Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                    ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
                    WebSocketOeServer webSocketOeServer = webSocketMap.get(clientWebsocketId);
                    ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                    if ((Objects.nonNull(examRecordStatusEnum)
                            && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                            && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                            && Objects.nonNull(webSocketOeServer.getRecordId())
                            && webSocketOeServer.getRecordId().longValue() == recordId.longValue()) {
//                        match.set(true);
                        Map map = new HashMap<>();
                        map.put(SystemConstant.RECORD_ID, recordId);
                        map.put(SystemConstant.BREACH_STATUS, FinishTypeEnum.valueOf(String.valueOf(mqDto.getProperties().get("type"))).getCode());
                        WebsocketDto websocketDto = new WebsocketDto(WebsocketTypeEnum.BREACH_STOP_EXAM.name(), map);
                        webSocketOeServer.sendMessage(websocketDto);
                        TEExamStudentLog teExamStudentLog = new TEExamStudentLog(mqDto.getType().name(), mqDto.getType().getCode(), mqDto.getType().getCode(), examStudentCacheBean.getStudentId(), examStudentId, recordId, mqDto.getType().getCode());
                        teExamStudentLogService.save(teExamStudentLog);
                    }
                }
            });
        } else if (Objects.equals(MqTagEnum.OE_IM_CLUSTERING.name(), tag)) {//点对点消息
            Long recordId = Long.parseLong(String.valueOf(mqDto.getBody()));
            String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
            if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId))) {
                Long examId = ExamRecordCacheUtil.getExamId(recordId);
                Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
                WebSocketOeServer webSocketOeServer = webSocketMap.get(clientWebsocketId);
                ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                if ((Objects.nonNull(examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                        && Objects.nonNull(webSocketOeServer.getRecordId())
                        && webSocketOeServer.getRecordId().longValue() == recordId.longValue()) {
//                    match.set(true);
                    Map<String, Object> prop = mqDto.getProperties();
                    Map map = new HashMap<>();
                    map.put(SystemConstant.RECORD_ID, recordId);
                    map.put("from", mqDto.getObjName());
                    map.put("type", prop.get("type"));
                    map.put("content", prop.get("content"));
                    WebsocketDto websocketDto = new WebsocketDto(WebsocketTypeEnum.INVIGILATE_NOTICE.name(), map);
                    webSocketOeServer.sendMessage(websocketDto);
                    TIeExamInvigilateNotice tIeExamInvigilateNotice = new TIeExamInvigilateNotice(examId, examActivityId, recordId, Long.parseLong(mqDto.getObjId()), examStudentId, MessageTypeEnum.valueOf(String.valueOf(prop.get("type")).toUpperCase()), String.valueOf(prop.get("content")));
                    tIeExamInvigilateNoticeService.saveOrUpdate(tIeExamInvigilateNotice);
                }
            }
        } else if (Objects.equals(MqTagEnum.OE_IM_BROADCASTING.name(), tag)) {//广播消息
            JSONArray jsonArray = JSONArray.parseArray(String.valueOf(mqDto.getBody()));
            Set<String> examStudentIdentitySet = jsonArray.toJavaObject(Set.class);
            log.info("examStudentIdentitySet:{}", JacksonUtil.parseJson(examStudentIdentitySet));
        } else if (Objects.equals(MqTagEnum.OE_LIVENESS_VERIFY.name(), tag)) {//监考强制活体验证
            Long recordId = Long.parseLong(String.valueOf(mqDto.getBody()));
            String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
            if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId))) {
                Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
                WebSocketOeServer webSocketOeServer = webSocketMap.get(clientWebsocketId);
                ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                if ((Objects.nonNull(examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                        && Objects.nonNull(webSocketOeServer.getRecordId())
                        && webSocketOeServer.getRecordId().longValue() == recordId.longValue()) {
//                    match.set(true);
                    Map map = new HashMap<>();
                    map.put(SystemConstant.RECORD_ID, recordId);
                    WebsocketDto websocketDto = new WebsocketDto(WebsocketTypeEnum.INVIGILATE_LIVENESS_VERIFY.name(), map);
                    webSocketOeServer.sendMessage(websocketDto);
                    TEExamStudentLog teExamStudentLog = new TEExamStudentLog(mqDto.getType().name(), mqDto.getType().getCode(), mqDto.getType().getCode(), examStudentCacheBean.getStudentId(), examStudentId, recordId, mqDto.getType().getCode());
                    teExamStudentLogService.save(teExamStudentLog);
                }
            }
        } else if (Objects.equals(MqTagEnum.OE_WEBSOCKET_MOBILE_ANSWER_READY.name(), tag)//移动端拍照/录音扫描完成
                || Objects.equals(MqTagEnum.OE_WEBSOCKET_MOBILE_ANSWER_UPLOAD.name(), tag)//移动端拍照/录音上传成功
                || Objects.equals(MqTagEnum.OE_WEBSOCKET_EXAM_STOP.name(), tag)//考试结束
                || Objects.equals(MqTagEnum.MONITOR_START.name(), tag)//监控开始
                || Objects.equals(MqTagEnum.MONITOR_STOP.name(), tag)//监控结束
                || Objects.equals(MqTagEnum.OE_WEBSOCKET_MOBILE_MONITOR_STATUS.name(), tag)) {//通知客户端移动端当前监控状态
            Long recordId = Long.parseLong(String.valueOf(mqDto.getBody()));
            String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
            Map<String, Object> prop = mqDto.getProperties();
            if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId))) {
                WebSocketOeServer webSocketOeServer = webSocketMap.get(clientWebsocketId);
                ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                if ((Objects.nonNull(examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                        && Objects.nonNull(webSocketOeServer.getRecordId())
                        && webSocketOeServer.getRecordId().longValue() == recordId.longValue()) {
//                    match.set(true);
                    WebsocketDto websocketDto = null;
                    switch (tag.toUpperCase()) {
                        case "OE_WEBSOCKET_MOBILE_ANSWER_READY":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_ANSWER_READY.name(), prop);
                            break;
                        case "OE_WEBSOCKET_MOBILE_ANSWER_UPLOAD":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_ANSWER_UPLOAD.name(), prop);
                            break;
                        case "OE_WEBSOCKET_EXAM_STOP":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.EXAM_STOP.name(), prop);
                            break;
                        case "MONITOR_START":
                            MonitorVideoSourceEnum source = MonitorVideoSourceEnum.valueOf((String) prop.get("source"));
                            MonitorStatusBean monitorStatusBean = ExamRecordCacheUtil.getMonitorStatusBean(recordId, source);
                            if (Objects.nonNull(monitorStatusBean)
                                    && Objects.nonNull(monitorStatusBean.getTimestamp())
                                    && mqDto.getTimestamp().longValue() >= monitorStatusBean.getTimestamp().longValue()) {
                                websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_MONITOR_START.name(), prop);
                            }
                            break;
                        case "MONITOR_STOP":
                            source = MonitorVideoSourceEnum.valueOf((String) prop.get("source"));
                            monitorStatusBean = ExamRecordCacheUtil.getMonitorStatusBean(recordId, source);
                            if (Objects.nonNull(monitorStatusBean)
                                    && Objects.nonNull(monitorStatusBean.getTimestamp())
                                    && mqDto.getTimestamp().longValue() >= monitorStatusBean.getTimestamp().longValue()) {
                                websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_MONITOR_STOP.name(), prop);
                            }
                            break;
                        case "OE_WEBSOCKET_MOBILE_MONITOR_STATUS":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_MONITOR_STATUS.name(), prop);
                            break;
                        default:
                            break;
                    }
                    webSocketOeServer.sendMessage(websocketDto);
                }
            }
        }
//        if (match.get()) {
            mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
            TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
            tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
            tmRocketMessageService.saveOrUpdate(tmRocketMessage);
            redisUtil.delete(key, mqDto.getId());
//        }
    }

    /**
     * oe mobile逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqOeMobileLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        String tag = mqDto.getTag();
//        AtomicBoolean match = new AtomicBoolean(false);
        ConcurrentHashMap<String, WebSocketMobileServer> webSocketMap = WebSocketMobileServer.getWebSocketMap();
        if (Objects.equals(MqTagEnum.EXAM_STOP.name(), tag)//考试退出
                || Objects.equals(MqTagEnum.EXAM_START.name(), tag)) {//考试开始
            Long recordId = Long.parseLong(String.valueOf(mqDto.getBody()));
            String clientWebsocketId = ExamRecordCacheUtil.getClientWebsocketId(recordId);
            if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId + "-" + MonitorVideoSourceEnum.MOBILE_FIRST.name()))) {
                WebSocketMobileServer webSocketMobileServer = webSocketMap.get(clientWebsocketId + "-" + MonitorVideoSourceEnum.MOBILE_FIRST.name());
                ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                if ((Objects.nonNull(examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                        && Objects.nonNull(webSocketMobileServer.getRecordId())
                        && webSocketMobileServer.getRecordId().longValue() == recordId.longValue()) {
//                    match.set(true);
                    WebsocketDto websocketDto = null;
                    switch (tag.toUpperCase()) {
                        case "EXAM_STOP":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.EXAM_STOP.name(), mqDto.getProperties());
                            break;
                        case "EXAM_START":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.EXAM_START.name(), mqDto.getProperties());
                            break;
                        default:
                            break;
                    }
                    webSocketMobileServer.sendMessage(websocketDto);
                }
            }
            if (Objects.nonNull(clientWebsocketId) && Objects.nonNull(webSocketMap.get(clientWebsocketId + "-" + MonitorVideoSourceEnum.MOBILE_SECOND.name()))) {
                WebSocketMobileServer webSocketMobileServer = webSocketMap.get(clientWebsocketId + "-" + MonitorVideoSourceEnum.MOBILE_SECOND.name());
                ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
                if ((Objects.nonNull(examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.PERSISTED, examRecordStatusEnum)
                        && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum))
                        && Objects.nonNull(webSocketMobileServer.getRecordId())
                        && webSocketMobileServer.getRecordId().longValue() == recordId.longValue()) {
//                    match.set(true);
                    WebsocketDto websocketDto = null;
                    switch (tag.toUpperCase()) {
                        case "EXAM_STOP":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.EXAM_STOP.name(), mqDto.getProperties());
                            break;
                        case "EXAM_START":
                            websocketDto = new WebsocketDto(WebsocketTypeEnum.EXAM_START.name(), mqDto.getProperties());
                            break;
                        default:
                            break;
                    }
                    webSocketMobileServer.sendMessage(websocketDto);
                }
            }
        }
//        if (match.get()) {
            mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
            TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
            tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
            tmRocketMessageService.saveOrUpdate(tmRocketMessage);
            redisUtil.delete(key, mqDto.getId());
//        }
    }
}
